@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 from __future__ import absolute_import, division
 
 """
@@ -8,6 +7,7 @@ Copyright (c) 2004-2007 Open Source Applications Foundation
 Author: Heikki Toivonen
 """
 
+import base64
 import hashlib
 import io
 import logging
@@ -30,7 +30,7 @@ class EVPTestCase(unittest.TestCase):
         pass
 
     def _pass_callback(self, *args):
-        return 'foobar'
+        return b'foobar'
 
     def _assign_rsa(self):
         rsa = RSA.gen_key(1024, 3, callback=self._gen_callback)
@@ -42,14 +42,15 @@ class EVPTestCase(unittest.TestCase):
         rsa = self._assign_rsa()
         rsa.check_key()
 
-    @unittest.skipIf(six.PY3,
-                     "test_pem hangs under python3 and is not yet fixed")
     def test_pem(self):
         rsa = RSA.gen_key(1024, 3, callback=self._gen_callback)
         pkey = EVP.PKey()
         pkey.assign_rsa(rsa)
-        self.assertNotEqual(pkey.as_pem(callback=self._pass_callback),
-                            pkey.as_pem(cipher=None))
+
+        result_w_callback = pkey.as_pem(callback=self._pass_callback)
+        result_wo_callback = pkey.as_pem(cipher=None)
+        self.assertNotEqual(result_w_callback, result_wo_callback)
+
         with self.assertRaises(ValueError):
             pkey.as_pem(cipher='noXX$$%%suchcipher',
                         callback=self._pass_callback)
@@ -67,14 +68,15 @@ class EVPTestCase(unittest.TestCase):
         self.assertEqual(len(der_blob), 160)
 
     def test_get_digestbyname(self):
-        self.assertEqual(m2.get_digestbyname('sha513'), None)
+        with self.assertRaises(EVP.EVPError):
+            m2.get_digestbyname('sha513')
         self.assertNotEqual(m2.get_digestbyname('sha1'), None)
 
     def test_MessageDigest(self):  # noqa
         with self.assertRaises(ValueError):
             EVP.MessageDigest('sha513')
         md = EVP.MessageDigest('sha1')
-        self.assertEqual(md.update('Hello'), 1)
+        self.assertEqual(md.update(b'Hello'), 1)
         self.assertEqual(util.octx_to_num(md.final()),
                          1415821221623963719413415453263690387336440359920)
 
@@ -85,7 +87,7 @@ class EVPTestCase(unittest.TestCase):
         # now run the same test again, relying on EVP.MessageDigest() to call
         # get_digestbyname() under the hood
         md = EVP.MessageDigest('sha1')
-        self.assertEqual(md.update('Hello'), 1)
+        self.assertEqual(md.update(b'Hello'), 1)
         self.assertEqual(util.octx_to_num(md.final()),
                          1415821221623963719413415453263690387336440359920)
 
@@ -152,8 +154,6 @@ class EVPTestCase(unittest.TestCase):
         with self.assertRaises(ValueError):
             EVP.hmac(b'key', b'data', algo='sha513')
 
-    @unittest.skipIf(six.PY3,
-                     "test_get_rsa hangs under python3 and is not yet fixed")
     def test_get_rsa(self):
         """
         Testing retrieving the RSA key from the PKey instance.
@@ -184,8 +184,6 @@ class EVPTestCase(unittest.TestCase):
         rsa3 = RSA.gen_key(1024, 3, callback=self._gen_callback)
         self.assertNotEqual(rsa.sign(digest), rsa3.sign(digest))
 
-    @unittest.skipIf(six.PY3,
-                     'FIXME test_load_key_string_pubkey hangs under python3')
     def test_load_key_string_pubkey(self):
         """
         Testing creating a PKey instance from PEM string.
@@ -270,6 +268,34 @@ class EVPTestCase(unittest.TestCase):
         with self.assertRaises(ValueError):
             util.pkcs7_pad('Hello', 256)
 
+    def test_pkey_verify_crash(self):
+        SIGN_PRIVATE = EVP.load_key('tests/rsa.priv.pem')
+        SIGN_PUBLIC = RSA.load_pub_key('tests/rsa.pub.pem')
+
+        def sign(data):
+            SIGN_PRIVATE.sign_init()
+            SIGN_PRIVATE.sign_update(data)
+            signed_data = SIGN_PRIVATE.sign_final()
+            return base64.b64encode(signed_data)
+
+        def verify(response):
+            signature = base64.b64decode(response['sign'])
+            data = response['data']
+            verify_evp = EVP.PKey()
+            # capture parameter on the following line is required by
+            # the documentation
+            verify_evp.assign_rsa(SIGN_PUBLIC, capture=False)
+            verify_evp.verify_init()
+            verify_evp.verify_update(data)
+            # m2.verify_final(self.ctx, sign, self.pkey)
+            fin_res = verify_evp.verify_final(signature)
+            return fin_res == 1
+
+        data = b"test message"
+        signature = sign(data)
+        res = {"data": data, "sign": signature}
+        self.assertTrue(verify(res))  # works fine
+        self.assertTrue(verify(res))  # segmentation fault in *verify_final*
 
 class CipherTestCase(unittest.TestCase):
     def cipher_filter(self, cipher, inf, outf):
